#include "Riostream.h"
#include "TNT2Event.h"
//using std::vector;

//oscillogram "event" data
ClassImp(SamplePoint);
ClassImp(Signal);

//==================================================================
SamplePoint::SamplePoint() : TObject()
{
  fPoint = 0;
}

SamplePoint::SamplePoint(Short_t sample) : TObject()
{
  fPoint = sample;
}

SamplePoint::SamplePoint(const SamplePoint& orig) : TObject(orig)
{
  //fPoint = orig.fPoint;
  *this = orig;
}

void SamplePoint::Print()
{
  cout << fPoint << ",";
}

void SamplePoint::Clear(Option_t * /*option*/)
{
  fPoint = 0;
}


//==================================================================
// Create a static TClonesArray at the first call to the constructor,
// then set the pointer fSamples to point to the static pointer. This
// guarantees that the TClonesArray is created only once.
//==================================================================
Signal::Signal() : TObject()
{
  // Create an signal Oscillogram
  fTriggerCounter = 0L;
  fNSamples       = 0;

  //the following allows Tree->Show(i) to work
  fSamples = new TClonesArray("SamplePoint",100);
}

Signal::Signal(const Signal &orig) : TObject(orig)
{
  //copy ctor
  fTriggerCounter = orig.fTriggerCounter;
  
  // create TClonesArray
  fSamples  = (TClonesArray*)orig.fSamples->Clone(); //is this the source of the leak ?
  fNSamples = orig.fNSamples;
}

Signal::~Signal()
{
  Clear();
  //fSamples->Delete(); //is this needed ?
}

void Signal::Clear(Option_t * /*option*/)
{
  //now
  fNSamples       =   0;
  fTriggerCounter =  0L;   //counter for this channel ?
  fSamples->Clear("C");    //clear ? TClonesArray
  //was
  //ClearArr();
}

void Signal::ClearArr()
{
  //cout << "signal clear : " << *option << endl;
  //Print();
  fNSamples       =   0;
  fTriggerCounter =  0L;   //counter for this channel ?
  //fSamples->Clear("C");    //clear ? TClonesArray
  fSamples->Delete();      //This deletes the samplepoints
}

Signal& Signal::operator=(const Signal &rhs)
{
  //copy
  this->fTriggerCounter = rhs.fTriggerCounter;
  this->fNSamples       = rhs.fNSamples;
  
  // create TClonesArray
  this->fSamples  = (TClonesArray*)rhs.fSamples->Clone();
  return *this;
}

SamplePoint*  Signal::AddASamplePoint()
{
  TClonesArray &Arr = *fSamples;
  SamplePoint  *A   =  new(Arr[fNSamples++]) SamplePoint();
  return A;
}

void Signal::AddSamplePoint(Short_t thisPoint)
{
  //cout << "Adding point " << thisPoint << " at sample " << fNSamples << endl;
  TClonesArray &samples = *fSamples;
  new(samples[fNSamples++]) SamplePoint(thisPoint);
}


void Signal::AddNSamplePoints(const TClonesArray& samplePoints)
{
  Int_t nPoints = samplePoints.GetLast()+1;

  TClonesArray &samples   = *fSamples;
  SamplePoint  *thisPoint = 0;
  Short_t       myPnt;

  for(Int_t n = 0; n < nPoints; n++) {
    thisPoint = (SamplePoint *) samplePoints.At(n);
    //thisPoint->Print();
    //myPnt = (SamplePoint *)(samplePoints.At(n))->GetSamplePoint();
    myPnt = thisPoint->GetSamplePoint();
    new(samples[fNSamples++]) SamplePoint(myPnt);
  }

  //samples.Delete();
}

void Signal::Print()
{
  //basic print-out
  cout << "    Signal "  << endl;
  cout << "       counter  "       << fTriggerCounter << endl;
  cout << "       nSamples "       << fNSamples       << endl;
  cout << "\t";
  
  TIter next(fSamples);
  //SamplePoint  *sample = new SamplePoint();
  SamplePoint  *sample = 0;
  for(Int_t m = 0; m < 10 && (sample = (SamplePoint*)next()); m++ ) {
    sample->Print();
  }
  cout << endl;
  // this delete causes errors in CheckSignalTree
  // Print() is only used for debugging so I should not worry if it introduces a leak
  //delete sample; 
}

