//====================================================	
//====================================================	
#ifndef __TNT2Event__		
#define __TNT2Event__
//====================================================
#include "TObject.h"
#include "TClonesArray.h"


//=============================================================
// Class defining the data available from an oscillogram file :
// The oscillograms for each channel are read out together and
// therefore have a common time => define a class for the
// individual signas and a class to contain the 1-4 signals.
// Merging many cards together will result in more than 4 signals.
//=============================================================


//===================================
// Classes to contain the signal data
//===================================

//-----------------------------------
// each sampled point
//-----------------------------------
class SamplePoint : public TObject {
 private:
  Short_t fPoint;
  
 public:
  SamplePoint();
  SamplePoint(const SamplePoint& orig);
  SamplePoint(Short_t sample);
  virtual ~SamplePoint() {Clear();}
  
  void  Clear(Option_t *option="");
  void  Print();
  
  void    SetSamplePoint(Short_t sample) { fPoint = sample;  }
  inline  Short_t GetSamplePoint() const { return fPoint; }
  
  ClassDef(SamplePoint, 1);
};


//==============================================
class Signal : public TObject {
 public:
  //could add Card Number here to allow merging of multiple cards
  UInt_t         fTriggerCounter;    //current event trigger number
  
  Int_t                fNSamples;    // number of samples in this oscillogram
  TClonesArray        *fSamples;      //->  //always allocated by default constructor
  //TClonesArray       fSamples;  //->  //always allocated by default constructor

  Signal();                                //constructor
  Signal(const Signal &);                  //copy-constructor
  ~Signal();                       //destructor

  Signal &operator=(const Signal &rhs); //copy-in

  TClonesArray  *GetArray()  const {return fSamples;}
  void           AddSamplePoint(Short_t sample);
  void           AddNSamplePoints(const TClonesArray& samplePoints);
  //SamplePoint*  Add(UShort_t sample);
  SamplePoint*    AddASamplePoint();

  void Print();           //basic print

  void Clear   (Option_t *option = "");
  void ClearArr();

  ClassDef(Signal, 1);
};
#endif
