//======================================
// Started as a macro to test Classes
//======================================
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
using namespace std;

#include "TNT2Event.h"
void TestEvent();
void test_Signal_Tree();
void test_FakeEvent_Tree();

//----------------------------------
//create a "main" when not using CINT
//----------------------------------
#ifndef __CINT__
#include "TROOT.h"
#include "TFile.h"
#include "TDirectoryFile.h"
#include "TBufferFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TObject.h"
#include "TObjectTable.h"
#include "TStopwatch.h"
#include "TRandom2.h"

//----------------------------------
int main() 
{
  TestEvent();
  cout << "in main" << endl;
  gObjectTable->Print();
  return 1;
}
#endif

//=================================================
//Create and fill a tree......
// A branch for the signal (a counter and a number of sample points)
// Woeks a trest : very minimal memeory leak
void test_Signal_Tree() 
{
  gObjectTable->Print();

  TFile* f = new TFile("TestSignalTree.root","RECREATE","");
  f->SetCompressionLevel(3);
  
  SamplePoint::Class()->IgnoreTObjectStreamer();   //Remove fUniqueId, fBits
  Signal::Class()->IgnoreTObjectStreamer();         //Remove fUniqueId, fBits

  //gObjectTable->Print();

  TTree* testTree0 = new TTree("testTree0","test0");
  Signal *nSig     = new Signal();  //on the heap
  testTree0->Branch("sigBranch","Signal",&nSig,8000,99);

  nSig->Clear();  //make sure !
  
  //Fill the tree for tests
  for(Int_t n = 0; n < 5000; n++) {
    nSig->Clear();
    nSig->fTriggerCounter = n;
    
    //add some sample data
    for(Int_t s = 1; s <= 2000; s++) {
      nSig->AddSamplePoint(n+s);      //the old way
    }
    testTree0->Fill(); //comment out to check for memory leaks [no leaks]
  }
  
  gObjectTable->Print();
  //ClearArr() instead of Clear() here to make sure SamplePoints are removed from memory
  //otherwise they do not ever get deleted
  nSig->ClearArr();

  f->Write();
  f->Close();
  delete nSig;
  //delete testTree0;
  delete f;

}

//======================================
// Fake an event : memory leak.... I have lost
// control of some TClonesArrays
void test_FakeEvent_Tree()
{
  cout << "===========================" << endl
       << "fake an OscEvent tree tests...."<<endl;
  
  TFile* f = new TFile("FakeEvent.root","RECREATE","");
  f->SetCompressionLevel(3);
  
  SamplePoint::Class()->IgnoreTObjectStreamer();   //Remove fUniqueId, fBits
  Signal::Class()->IgnoreTObjectStreamer();         //Remove fUniqueId, fBits

  
  TTree* testTree1 = new TTree("testTree1","test1");
  
  //fake an event.........ideally I would want an
  //"event" class that contains header info and "Signal" TClonesArray
  ULong64_t thisEventNumber;
  ULong64_t thisTimeStamp;

  testTree1->Branch("fTimeStamp",&thisTimeStamp,800);
  testTree1->Branch("fEventNumber",&thisEventNumber,800);
  
  cout << "Create the TClonesArrays " << endl;
  
  TClonesArray *sArray2 = new TClonesArray("Signal",   4);
  //TClonesArray &sArr    = *sArray2;
  testTree1->Branch("sBranch","TClonesArray", &sArray2,1000,99);
  
  //gObjectTable->Print(); //2 TClonesArray 
  
  Int_t      nTrig = 0;
  //Signal* newSignal = 0;
  for(Int_t event = 0; event < 5; event++) {
    
    thisEventNumber = event;
    thisTimeStamp   = 100*event + gRandom->Integer(100);
    
    //add some sample data for each signal : memory leak of n*event TClonesArrays.....
    //gObjectTable->Print(); //2 TClonesArray for EVENT 0 (+4 for each "event")
    
    for(Int_t n = 0; n < 4; n++) {
      
      //newSignal = new((*sArray2)[n]) Signal();   //test2
      //newSignal->fTriggerCounter = nTrig++;
      
      //for(Int_t s = 0; s < 1000; s++) {
      //SamplePoint *thisPoint = newSignal->AddASamplePoint();
      //thisPoint->SetSamplePoint(2*event+n+s); //test1-4
      //}
      
      new((*sArray2)[n]) Signal();      //test5
      ((Signal *) sArray2->At(n))->fTriggerCounter = nTrig++;
      
      //gObjectTable->Print();  //+1 TClonesArray on the heap for each pass
      
      for(Int_t s = 0; s < 1000; s++) {
	((Signal *) sArray2->At(n))->AddSamplePoint(2*event+n+s); //test5
      }
    }
    //Fill the tree for this event
    testTree1->Fill(); 
    
    //Or, must I do this
    for(Int_t n = 0; n <= sArray2->GetLast(); n++) {
      ((Signal *) sArray2->At(n))->ClearArr();      //otherwise SamplePoints left
    }
    //newSignal->ClearArr();
    //
    sArray2->Clear("C");
  }

  f->Write();
  f->Close();  
  
  //gObjectTable->Print();

  //sArray2->Delete();
  //gObjectTable->Print();
}

//----------------------------------
//----------------------------------
void TestEvent()
{
#ifdef __CINT__
  gSystem->Load("./libTNT2Event.so");
#endif
  cout << endl
       << "===========================" << endl
       << "test some trees....."<<endl;
  
  //test_Signal_Tree();
  //gObjectTable->Print();

  test_FakeEvent_Tree();
  cout << "Back in TestEvent" << endl;
  
  gObjectTable->Print();
  return;
}
